**********************************************
****Chapter 7: Growth curve models *****
**********************************************


use http://www.stata-press.com/data/mlmus3/asian, clear

** 7.2.1 Observed growth trajectories

label define g 1 "Boy" 2 "Girl"
label values gender g
sort id age

graph twoway (line weight age, connect(ascending)), ///
 by(gender) xtitle(Age in years) ytitle(Weight in Kg)

*** 7.3 Models for nonlinear growth

** 7.3.1 Polynomial models
 
* fitting the models

recode gender 2=1 1=0, generate(girl)
generate age2 = age^2
xtmixed weight girl age age2 || id: age, covariance(unstructured) mle
estimates store rc

xtmixed weight girl age age2 || id: age age2, covariance(unstructured) mle
estimates restore rc

* predicting the mean trajectory

twoway (function Weight=_b[_cons]+_b[age]*x+_b[age2]*x^2,     ///
   range(0.1 2.6)), xtitle(Age in years) ytitle(Weight in Kg) ///
   yscale(range(0 20)) ylab(0(5)20)
   
estat recovariance

scalar var1 = exp(2*[lns1_1_1]_cons)
scalar var2 = exp(2*[lns1_1_2]_cons)
scalar cov = tanh([atr1_1_1_2]_cons)*sqrt(var1*var2)

twoway (function Weight=_b[_cons]+_b[age]*x+_b[age2]*x^2,               /// 
        range(0.1 2.6) lwidth(medium))                                  /// 
        (function upper=_b[_cons]+_b[age]*x+_b[age2]*x^2                /// 
        +1.96*sqrt(var1+2*cov*x+var2*x^2), range(0.1 2.6) lpatt(dash))  /// 
        (function lower=_b[_cons]+_b[age]*x+_b[age2]*x^2                /// 
        -1.96*sqrt(var1+2*cov*x+var2*x^2), range(0.1 2.6) lpatt(dash)), /// 
        legend(order(1 "Mean" 2 "95\% range")) xtitle(Age in years)     /// 
        ytitle(Weight in kg) yscale(range(0 20)) ylab(0(5)20)


* Predicting trajectories for individual children

predict traj, fitted

twoway (scatter weight age) (line traj age, sort) if girl==1, ///
 by(id, compact legend(off))

*** 7.3.2 Piecewise linear models

* Fitting the models

disp _N
set obs 201
replace age = 0.4 in 199
replace age = 0.9 in 200
replace age = 1.5 in 201
mkspline ages1 .4 ages2 .9 ages3 1.5 ages4 = age


twoway (line ages1 age, sort) (line ages2 age, sort) ///
   (line ages3 age, sort) (line ages4 age, sort),    ///
   xscale(range(0 2.5)) xlabel(0 .4 .9 1.5)

   
xtmixed weight girl ages1 ages2 ages3 ages4 ///
   || id: age, covariance(unstructured) mle


* Predicting the mean trajectory

replace girl = 0 in 199/201
predict mean, xb
scalar var1 = exp(2*[lns1_1_1]_cons)
scalar var2 = exp(2*[lns1_1_2]_cons)
scalar cov = tanh([atr1_1_1_2]_cons)*sqrt(var1*var2)
generate upper = mean + 1.96*sqrt(var1+2*cov*age+var2*age^2) 
generate lower = mean - 1.96*sqrt(var1+2*cov*age+var2*age^2) 

twoway (line mean age, sort lwidth(medium)) ///
       (line lower age, sort lpatt(dash)) ///
        (line upper age, sort lpatt(dash)) if girl==0, ///
		legend(order(1 "Mean" 2 "95% range")) xtitle(Age in years) ///
		ytitle(Weight in kg) xtick(.4 .9 1.5, grid) ylabel(,nogrid) ///
		yscale(range(0 20)) ylab(0(5)20)
		

*** 7.4 Two-stage formulation

generate age_girl = age*girl
xtmixed weight girl age_girl age age2 || id: age, covariance(unstructured) mle

*** 7.5 Heteroskedasticity

** 7.5.1 Heteroskedasticity at level 1

xtmixed weight girl age age2 || id: age, covariance(unstructured) mle ///
   residuals(independent, by(gender))
lrtest rc .

** 7.5.2 Heteroskedasticity at level 2

generate boy = 1-girl
generate age_boy = age*boy

xtmixed weight girl age age2 || id: age_boy boy, noconstant covariance(unstructured) ///
   || id: age_girl girl, noconstant covariance(unstructured) mle
lrtest rc .

*** 7.6 How does reading improve from kindergarten through third grade?

** 7.7.1 Estimation using sem

use http://www.stata-press.com/data/mlmus3/reading, clear

misstable patterns read*

graph box read0 read1 read2 read3, ascategory intensity(0) medtype(line)

sem (read0 <- L1@1 L2@0 ) ///
    (read1 <- L1@1 L2@1 ) ///
    (read2 <- L1@1 L2@2 ) ///
    (read3 <- L1@1 L2@3 ) , means(L1 L2) nocons method(mlmv)

sem (L1 -> read0@1 read1@1 read2@1 read3@1) ///
    (L2 -> read1@1 read2@2 read3@3), means(L1 L2) noconstant method(mlmv) 

predict l1 l2, latent

** 7.7.2 Estimation using xtmixed

reshape long read math age, i(id) j(grade)

quietly xtset id grade
drop if read>=.
xtdescribe 

egen mn_read = mean(read), by(grade)

twoway (connected mn_read grade, sort), xtitle(Grade) ///
   ytitle(Mean reading score)
   

/*
graph box read, over(grade) ytitle(Reading score) intensity(0) medtype(line)
*/
  
xtmixed read grade || id: grade, covariance(unstructured) mle ///
   variance residual(independent, by(grade)) 

predict fixed, xb
twoway (connected mn_read grade, sort lpatt(solid)) ///
        (connected fixed grade, sort lpatt(dash)), xtitle(Grade) ///
      ytitle(Mean reading score) legend(order(1 "Raw mean" 2 "Fitted mean"))
	  
predict zeta2 zeta1, reffects
generate eta1 = _b[_cons] + zeta1
generate eta2 = _b[grade] + zeta2

*assert abs(l1-zeta1)<1e-3
*assert abs(l2-zeta2)<1e-3


